module Main where

import Eval
import Parser

env :: Env
env = [ ("foo", 42)
      , ("bar", 0)
      , ("x", sqrt 2)
      , ("pi", pi)
      , ("e", exp 1)
      ]

fromRight :: Either a b -> b
fromRight (Right b) = b

test1 :: Expr
test1 = fromRight $ parseThis exprParser "( ( ( foo * 2 ) / x ) / x)"
expected1 = Right 42
--  runEval (eval test1) env --> Right 42
test2 :: Expr
test2 = fromRight $ parseThis exprParser "( ( foo * bar) + quux )"
expected2 = Left "Unbound variable!"
--  runEval (eval test2) env --> Left "Unbound variable!"
test3 :: Expr
test3 = fromRight $ parseThis exprParser "( ( foo * bar) + e )"
expected3 = Right $ exp 1
--  runEval (eval test3) env --> Right 2.71...
test4 :: Expr
test4 = fromRight $ parseThis exprParser "( ( foo / bar) + e )"
expected4 = Left "Division by 0!"
--  runEval (eval test4) env --> Left "Division by 0!"

check :: Expr -> ResultOrError Double -> Bool
check expr res = let e = runEval (eval expr) env
                     compare a b = abs (a - b) < 0.001
                  in case (compare <$> e <*> res) of
                        Left e  -> Left e == res
                        Right b -> b

main :: IO ()
main = do
    putStrLn "Test1:"
    print $ check test1 expected1
    putStrLn "Test2:"
    print $ check test2 expected2
    putStrLn "Test3:"
    print $ check test3 expected3
    putStrLn "Test4:"
    print $ check test4 expected4
  
