
module Parser where

import Text.Parsec

import Eval

type Parser = Parsec String String Expr

exprParser :: Parser
exprParser = do spaces
                e <- litParser <|> varParser <|> binParser
                spaces
                return e

litParser :: Parser
litParser = do x <- many1 digit
               y <- option "" (char '.' >> many1 digit)
               let suffix = if y == ""
                              then ""
                              else "." ++ y
               return $ Lit $ read $ x ++ suffix

varParser :: Parser
varParser = do v <- many1 letter
               return $ Var v

binParser :: Parser
binParser = do char '('
               spaces
               l <- exprParser
               spaces
               op <- oneOf "+-*/"
               spaces
               r <- exprParser
               spaces
               char ')'
               let constr = case op of
                          '+' -> Add
                          '-' -> Sub
                          '*' -> Mul
                          '/' -> Div
               return $ constr l r


parseThis :: Parser -> String -> Either ParseError Expr
parseThis p = runParser p "" ""
